<?php


namespace App\Controller;


use Doctrine\ORM\EntityManagerInterface;
use phpDocumentor\Reflection\Types\Self_;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/number-of-sold-by-region", name="number_of_sold_by_region")
 */
class NumberOfSoldByRegion extends AbstractController
{
    /**
     * Entity Manager
     */
    private $em;
    public function __construct(EntityManagerInterface $em) {
        $this->em = $em;
    }

    private static $REGIONS = [
        'Auvergne-Rhône-Alpes' => [3,15,43,63,1,7,26,38,42,69,73,74],
        'Normandie' => [14,50,61,27,76],
        'Bourgogne-Franche-Comté' => [21,58,71,89,25,39,70,90],
        'Bretagne' => [22,29,35,56],
        'Centre-Val de Loire' => [18,28,36,37,41,45],
        'Grand Est' => [8,10,51,52,54,55,57,88,67,68],
        'Ile-de-France' => [75,77,78,91,92,93,94,95],
        'Occitanie' => [11,30,34,48,66,9,12,31,32,46,65,81,82],
        'Hauts-de-France' => [59,62,2,60,80],
        'Pays de la Loire' => [44,49,53,72,85],
        'Nouvelle-Aquitaine' => [16,17,79,86,19,23,87,24,33,40,47,64],
        'Provence-Alpes-Côte-d\'Azur' => [4,5,6,13,83,84],
        'Guadeloupe' => [971],
        'Martinique' => [972],
        'Guyane' => [973],
        'La Réunion' => [974],
        'Mayotte' => [976]
    ];

    public function __invoke(Request $request) {
        $counts_by_departement = $this->em->getRepository('App:ValeursFonciere')->getNumberOfItemsSoldGroupedByDepartement();
        $counts_by_region = [];
        foreach ($counts_by_departement as $count_by_departement){
            $count_by_departement['code_departement'] = intval($count_by_departement['code_departement']);
            $count_by_departement['count_by_departement'] = intval($count_by_departement['count_by_departement']);
            $region = $this->getRegionByDepartement($count_by_departement['code_departement']);
            if($region){
                if(!key_exists($region, $counts_by_region)){
                    $counts_by_region[$region] = 0;
                }
                $counts_by_region[$region] += $count_by_departement['count_by_departement'];
            }
        }
        $counts_by_region_fixed = [];
        $total_count = 0;
        foreach ($counts_by_region as $region_name => $count_by_region){
            $counts_by_region_fixed[] = [
                'region' => $region_name,
                'count_by_region' => $count_by_region
            ];
            $total_count += $count_by_region;
        }
        foreach ($counts_by_region_fixed as &$count_by_region){
            $count_by_region['count_by_region'] = ($count_by_region['count_by_region'] / $total_count)*100;
        }
        $res = new JsonResponse(
            $counts_by_region_fixed,
            Response::HTTP_OK
        );
        return $res;
    }

    private function getRegionByDepartement($departement){
        foreach (self::$REGIONS as $region_name => $region_departements){
            if(array_search($departement,$region_departements,true) !== false){
                return $region_name;
            }
        }
        return false;
    }
}
