<?php

namespace App\DataFixtures;

use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Common\Persistence\ObjectManager;
use League\Csv\Reader;
use App\Entity\ValeursFonciere;
use League\Csv\Statement;

class AppFixtures extends Fixture
{
    public function load(ObjectManager $manager)
    {
        // Arrêter les Loggers afin de ne pas surcharger la mémoire
        $manager->getConnection()->getConfiguration()->setSQLLogger(null);

        // Sauter la première ligne (Ligne des titres)
        $offset = 1;
        // Nombre des records après lequel il faut sauvegarder
        $recordsLimitByLoop = 100;

        $fileName_partLeft = 'valeursfoncieres-201';
        $fileName_partRight = '.txt';

        // On a 5 fichiers de données (de l'année 15 jusqu'à 19)
        for($fileName_partMiddle=5; $fileName_partMiddle <= 9; $fileName_partMiddle++){
            // Le nom des fichiers de données sont composés par trois parties
            // Premier partie: Le nom jusqu'à l'année 'valeursfoncieres-201'
            // Deuxième partie: L'année (de 5 à 9)
            // Troisième partie: L'extension '.txt'
            $fileName = $fileName_partLeft . $fileName_partMiddle . $fileName_partRight;

            var_dump("started file " . $fileName);

            $csv = Reader::createFromPath('%kernel.root.dir%/../public/Donnees/'.$fileName);
            $csv->setDelimiter("|");
            $stmt = (new Statement())->offset($offset);
            $records = $stmt->process($csv);

            $numRecordsRead = 0;

            foreach ($records as $row) {
                $codeTypeLocal = intval($row[35]);
                if($codeTypeLocal == 1 || $codeTypeLocal == 2) {
                    // Créer un nouveau "Entity"
                    $valeursFonciere = new ValeursFonciere();
                    $valeursFonciere->setDateMutation(\DateTime::createFromFormat("d/m/Y", $row[8]));
                    $valeursFonciere->setNatureMutation($row[9]);
                    $valeursFonciere->setValeurFonciere(floatval(str_replace(',', '.', $row[10])));
                    $valeursFonciere->setCommune($row[17]);
                    $valeursFonciere->setCodeDepartement(intval($row[18]));
                    $valeursFonciere->setCodeCommune(intval($row[19]));
                    $valeursFonciere->setCodeTypeLocal(intval($row[35]));
                    $valeursFonciere->setTypeLocal($row[36]);
                    $valeursFonciere->setSurfaceReelleBati(floatval(str_replace(',', '.', $row[38])));
                    $manager->persist($valeursFonciere);

                    $numRecordsRead++;

                    // On fait un "Flush" chaque 100 records
                    if (($numRecordsRead % $recordsLimitByLoop) == 0) {
                        $manager->flush();
                        $manager->clear();
                    }
                }
            }

            $manager->flush();
            var_dump("Num of records " . $numRecordsRead);
        }
    }
}
