<?php


namespace App\Tests;

use ApiPlatform\Core\Bridge\Symfony\Bundle\Test\ApiTestCase;
use App\Entity\ValeursFonciere;
use Symfony\Component\HttpClient\HttpClient;

class ValeursFonciereTest extends ApiTestCase
{
    // Running the tests
    // docker-compose exec php bin/phpunit --debug
    public function testMeanPriceForSquaredMeter(): void {
        $this->setName("Testing /mean-price-for-squared-meter route");
        $response = self::createClient()->request('GET', '/mean-price-for-squared-meter');
        $this->assertResponseIsSuccessful();
        $this->assertResponseHeaderSame('content-type', 'application/json');
        $this->assertStringContainsString('"year":2019', $response->getContent());
        $this->assertMatchesJsonSchema([
            "type" => "array",
            "items" => [
                "type" => "object",
                "properties" => [
                    "year" => ["type"=>"number"],
                    "mean_metre_price"=> ["type"=>"number"]
                ]
            ]
        ]);
    }

    public function testNumberOfSoldByParams1(): void {
        $this->setName("Testing /number-of-sold-by-params (test 1)");
        $interval = "year";
        $startDate = "2016-01-01";
        $endDate = "2018-12-31";
        $response = self::createClient()->request('GET', "/number-of-sold-by-params/{$interval}/{$startDate}/{$endDate}");
        $this->assertResponseIsSuccessful();
        $this->assertResponseHeaderSame('content-type', 'application/json');
        $this->assertStringContainsString('"year":2018', $response->getContent());
        $this->assertStringNotContainsString('"year":2019', $response->getContent());
        $this->assertMatchesJsonSchema([
            "type" => "array",
            "items" => [
                "type" => "object",
                "properties" => [
                    "year" => ["type"=>"number"],
                    "number_of_sold"=> ["type"=>"number"]
                ]
            ]
        ]);
    }

    public function testNumberOfSoldByParams2(): void {
        $this->setName("Testing /number-of-sold-by-params (test 2)");
        $interval = "month";
        $startDate = "2016-01-01";
        $endDate = "2018-11-30";
        $response = self::createClient()->request('GET', "/number-of-sold-by-params/{$interval}/{$startDate}/{$endDate}");
        $this->assertResponseIsSuccessful();
        $this->assertResponseHeaderSame('content-type', 'application/json');
        $this->assertStringContainsString('"year":2018', $response->getContent());
        $this->assertStringContainsString('"year":2018,"month":11', $response->getContent());
        $this->assertStringNotContainsString('"year":2018,"month":12', $response->getContent());
        $this->assertStringNotContainsString('"year":2019', $response->getContent());
        $this->assertMatchesJsonSchema([
            "type" => "array",
            "items" => [
                "type" => "object",
                "properties" => [
                    "year" => ["type"=>"number"],
                    "month" => ["type"=>"number"],
                    "number_of_sold"=> ["type"=>"number"]
                ]
            ]
        ]);
    }

    public function testNumberOfSoldByRegion(): void {
        $this->setName("Testing /number-of-sold-by-region");
        $response = self::createClient()->request('GET', "/number-of-sold-by-region");
        $this->assertResponseIsSuccessful();
        $this->assertResponseHeaderSame('content-type', 'application/json');
        $this->assertStringContainsString('"region":"Normandie"', $response->getContent());
        $this->assertMatchesJsonSchema([
            "type" => "array",
            "items" => [
                "type" => "object",
                "properties" => [
                    "region" => ["type"=>"string"],
                    "count_by_region"=> ["type"=>"number"]
                ]
            ]
        ]);
    }
}
