import React, { Component } from 'react'
import GraphsHeader from './GraphsHeader'
import './styles.css';
import * as d3 from "d3";

export class GraphsSoldItemsAccordingToTimeInterval extends Component {


  constructor(props) {
    super(props);
    this.state = {
      isLoading: false,
      error: null,
      data: [],
      searchExecuted: false,
      searchData:{
        interval: "month",
        startDate: "2016-01-01",
        endDate: "2016-06-01"
      }
    }
    this.searhFormSubmitted = this.searhFormSubmitted.bind(this);
    this.elementChanged = this.elementChanged.bind(this);
    this.container = React.createRef();
    this.svg = React.createRef();
    this.intervalSelect = React.createRef();
    this.startDateInput = React.createRef();
    this.endDateInput = React.createRef();
  }

  render() {
    return (
      <div>
          <GraphsHeader />
          <div ref={this.container} className="container pt-5">
            <h1 class="text-center">
              Number of sold items in the selected time interval
            </h1>
            {this.getComponentBody()}
          </div>
      </div>
    )
  }

  getComponentBody(){
    const {isLoading, error, data } = this.state;
    if(error){
      return (
        <div>
          {this.getSearchFields()}
          <div className="alert alert-danger">{error.error}</div>
        </div>);
    }
    if(isLoading){
      return (<div className="alert alert-success">Loading...</div>);
    }
    return(
      <div>
        {this.getSearchFields()}
        {this.getGraph()}
      </div>
    );
  }

  getSearchFields(){
    return (
      <form className="search-form" onSubmit={this.searhFormSubmitted}>
        <div className="form-row">
          <div className="form-group col-md-4">
            <label for="interval">Interval</label>
            <select className="form-control" id="interval" ref={this.intervalSelect} value={this.state.searchData.interval} onChange={this.elementChanged}>
              <option value="day">Day</option>
              <option value="month" selected>Month</option>
              <option value="year">Year</option>
            </select>
          </div>
          <div className="form-group col-md-4">
            <label for="start_date">Start Date</label>
            <input type="date" className="form-control" id="start_date" ref={this.startDateInput} placeholder="e.g 2018-12-09" value={this.state.searchData.startDate} onChange={this.elementChanged} />
          </div>
          <div className="form-group col-md-4">
            <label for="end_date">Start Date</label>
            <input type="date" className="form-control" id="end_date" ref={this.endDateInput} placeholder="e.g 2018-12-09" value={this.state.searchData.endDate} onChange={this.elementChanged} />
          </div>
        </div>
        <div className="form-row">
          <div className="form-group col-md-4 offset-md-8">
            <button type="submit" value="Submit" className="btn btn-success form-control">Submit</button>
          </div>
        </div>
      </form>
    );
  }

  getGraph(){
    return (
      <div>
        <svg ref={this.svg} class="graph2"></svg>
      </div>
    );
  }

  searhFormSubmitted(event){
    event.preventDefault();
    let interval = this.intervalSelect.current.value;
    let startDate = this.startDateInput.current.value;
    let endDate = this.endDateInput.current.value;
    let d1 = new Date(startDate);
    let d2 = new Date(endDate);
    if(d1.getTime() > d2.getTime() || this.datesSameDay(d1,d2)){
      this.setState({ error: {error: "Dates are not correct !"}})
    }else{
      this.getData(interval, startDate, endDate);
    }
  }

  datesSameDay(date1, date2){
    return date1.getFullYear() === date2.getFullYear() &&
    date1.getMonth() === date2.getMonth() &&
    date1.getDate() === date2.getDate();
  }

  elementChanged() {
    let interval = this.intervalSelect.current.value;
    let startDate = this.startDateInput.current.value;
    let endDate = this.endDateInput.current.value;
    this.setState({
      searchData:{
        interval: interval,
        startDate: startDate,
        endDate: endDate
      },
      justAnInputChange: true
    });
  }

  createChart(){
    let data = this.state.data;
    let interval = this.state.searchData.interval;
    var margin = {top: 20, right: 20, bottom: 70, left: 60},
      width = this.container.current.clientWidth - margin.left - margin.right,
      height = 400 - margin.top - margin.bottom,
      parseDate,
      dateFormat,
      xAxisFormat;
    if(this.state.searchData.interval === "year"){
      dateFormat = "%Y";
      xAxisFormat = "%Y";
    }else if (this.state.searchData.interval === "month"){
      dateFormat = "%Y-%m";
      xAxisFormat = "%Y %B";
    }else{
      dateFormat = "%Y-%m-%d";
      xAxisFormat = "%Y %B";
    }
    parseDate = d3.timeParse(dateFormat);
    var x = d3.scaleBand()
          .range([0, width])
          .padding(0.1);
    var y = d3.scaleLinear()
          .range([height, 0]);
    var svg = d3.select(this.svg.current)
          .attr("width", width + margin.left + margin.right)
          .attr("height", height + margin.top + margin.bottom)
        .append("g")
          .attr("transform",
                "translate(" + margin.left + "," + margin.top + ")");
    // format the data
    var ticksArray = [];
    var lastMonth = 0;
    data.forEach(function(d) {
      let d_date = "";
      if(interval == "year"){
        d_date = d.year + "";
      }else if (interval == "month"){
        d_date = `${d.year}-${(d.month< 10?"0":"")}${d.month}`;
      }else{
        d_date = `${d.year}-${(d.month< 10?"0":"")}${d.month}-${(d.day< 10?"0":"")}${d.day}`;
        if(lastMonth !== d.month){
          ticksArray.push(parseDate(d_date));
          lastMonth = d.month;
        }
      }
      d.date = parseDate(d_date);
      d.number_of_sold = +d.number_of_sold;
    });

    // Scale the range of the data in the domains
    x.domain(data.map(function(d) { return d.date; }));
    y.domain([0, d3.max(data, function(d) { return d.number_of_sold; })]);

    // append the rectangles for the bar chart
    svg.selectAll(".bar")
        .data(data)
      .enter().append("rect")
        .attr("class", "bar")
        .attr("x", function(d) { return x(d.date); })
        .attr("width", x.bandwidth())
        .attr("y", function(d) { return y(d.number_of_sold); })
        .attr("height", function(d) { return height - y(d.number_of_sold); });

    // add the x Axis
    var x_axis = d3.axisBottom(x).tickFormat(d3.timeFormat(xAxisFormat));
    if(interval == "day"){
      x_axis = x_axis.tickValues(ticksArray);
    }
    svg.append("g")
        .attr("transform", "translate(0," + height + ")")
        .call(x_axis)
        .selectAll("text")
        .style("text-anchor", "end")
        .attr("dx", "-.8em")
        .attr("dy", "-.55em")
        .attr("transform", "rotate(-45)" );

    // add the y Axis
    svg.append("g")
        .call(d3.axisLeft(y));

  }

  getData(interval, startDate, endDate){
    this.setState({
      isLoading: true,
      searchExecuted: true,
      searchData:{
        interval: interval,
        startDate: startDate,
        endDate: endDate
      },
      justAnInputChange: false
    })
    fetch(`https://${document.domain}:8443/number-of-sold-by-params/${interval}/${startDate}/${endDate}`)
      .then(response => {
        if (response.ok) {
          const data = response.json()
          return data;
        } else {
          throw new Error('Something went wrong ...');
        }
      })
      .then(data => {
        this.setState({data: data, isLoading: false });
      })
      .catch(error => this.setState({ error: error, isLoading: false }));
  }

  componentDidUpdate(){
    if(this.state.isLoading === false && this.state.error == null && this.state.searchExecuted && !this.state.justAnInputChange){
      this.createChart();
    }
  }
}

export default GraphsSoldItemsAccordingToTimeInterval
