import React, { Component } from 'react'
import GraphsHeader from './GraphsHeader'
import './styles.css';
import * as d3 from "d3";

export class GraphsSoldItemsByRegion extends Component {
  render() {
    return (
      <div>
          <GraphsHeader />
      </div>
    )
  }


  constructor(props) {
    super(props);
    this.state = {
      isLoading: false,
      error: null,
      data: []
    }
    this.container = React.createRef();
    this.svg = React.createRef();
  }

  render() {
    return (
      <div>
          <GraphsHeader />
          <div ref={this.container} className="container pt-5">
            <h1 class="text-center">
              Number of sold items by region
            </h1>
            {this.getComponentBody()}
          </div>
      </div>
    )
  }

  getComponentBody(){
    const {isLoading, error, data } = this.state;
    if(error){
      return (<div className="alert alert-danger">{error.error}</div>);
    }
    if(isLoading){
      return (<div className="alert alert-success">Loading...</div>);
    }
    return this.getGraph();
  }

  getGraph(){
    return (<svg ref={this.svg} class="graph1" style={{margin: "0 auto", display:"block", overflow:"visible"}}></svg>);
  }

  createChart(){
    let data = this.state.data;
    let sum = Object.values(data).reduce((acc, current) => acc + current.count_by_region, 0);
    let average = sum / Object.values(data).length;
    let others = 0;
    let newData = [];
    for(let i in data){
      if(data[i].count_by_region < 4){
        others += data[i].count_by_region;
      }else{
        newData.push(data[i]);
      }
    }
    data = newData;
    if(others > 0){
      data.push({
        region: "...",
        count_by_region: others
      });
    }
    var width = 500;
    var height = 500;
    var radius = Math.min(width, height) / 2;
    var donutWidth = 75;
    var r = Math.min(width, height) / 2;

    var svg = d3.select(this.svg.current)
      .attr('width', width)
      .attr('height', height)
      .append('g')
      .attr('transform', 'translate(' + (width / 2) +
        ',' + (height / 2) + ')');

    var arc = d3.arc()
      .innerRadius(radius - donutWidth)
      .outerRadius(radius);

    var pie = d3.pie()
      .value(function(d) { return d.count_by_region; })
      .sort(null);

    var path = svg.selectAll('path')
      .data(pie(data))
      .enter()
      .append('path')
      .attr('d', arc)
      .attr('fill', function(d, i) {
        return "#"+((1<<24)*Math.random()|0).toString(16);
      });

  svg
    .selectAll('allPolylines')
    .data(pie(data))
    .enter()
    .append('polyline')
      .attr("stroke", "black")
      .style("fill", "none")
      .attr("stroke-width", 1)
      .attr('points', function(d) {
        var posA = arc.centroid(d)
        var posB = arc.centroid(d);
        var posC = arc.centroid(d);
        var midangle = d.startAngle + (d.endAngle - d.startAngle) / 2;
        posC[0] = radius * 0.95 * (midangle < Math.PI ? 1 : -1);
        return [posA, posB, posC];
      })

    svg
      .selectAll('allLabels')
      .data(pie(data))
      .enter()
      .append('text')
      .text( function(d) {return d.data.region } )
      .attr('transform', function(d) {
          var pos = arc.centroid(d);
          var midangle = d.startAngle + (d.endAngle - d.startAngle) / 2
          pos[0] = radius * 0.99 * (midangle < Math.PI ? 1 : -1);
          return 'translate(' + pos + ')';
      })
      .style('text-anchor', function(d) {
          var midangle = d.startAngle + (d.endAngle - d.startAngle) / 2
          return (midangle < Math.PI ? 'start' : 'end')
      });

    svg
      .selectAll('allLabels')
      .data(pie(data))
      .enter()
      .append('text')
      .text( function(d) {return d.data.count_by_region.toFixed(1) + "%" } )
      .attr('transform', function(d) {
          var pos = arc.centroid(d);
          var numChar = d.data.count_by_region.toFixed(1).toString().length + 2;
          var move = numChar * 7.5 / 2;
          return `translate(${pos[0] - move},${pos[1] - 5})`;
      });

  }


  componentDidMount() {
    this.setState({ isLoading: true })
    fetch(`https://${document.domain}:8443/number-of-sold-by-region`)
      .then(response => {
        if (response.ok) {
          const data = response.json()
          return data;
        } else {
          throw new Error('Something went wrong ...');
        }
      })
      .then(data => {
        this.setState({data: data, isLoading: false });
      })
      .catch(error => this.setState({ error: error, isLoading: false }));
  }

  componentDidUpdate(){
    if(this.state.isLoading === false || this.error == null){
      this.createChart();
    }
  }
}

export default GraphsSoldItemsByRegion
